@@ -1,6 +1,6 @@
 # Copyright 2010 Jacob Kaplan-Moss
 
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,8 +23,8 @@ import contextlib
 import hashlib
 import os
 
+from manilaclient.common import cliutils
 from manilaclient import exceptions
-from manilaclient.openstack.common import cliutils
 from manilaclient import utils
 
 
