@@ -1,5 +1,5 @@
 # Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -86,6 +86,21 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
         }
         return (200, {}, body)
 
+    def get_availability_zones(self):
+        availability_zones = {
+            "availability_zones": [
+                {"id": "368c5780-ad72-4bcf-a8b6-19e45f4fafoo",
+                 "name": "foo",
+                 "created_at": "2016-07-08T14:13:12.000000",
+                 "updated_at": "2016-07-08T15:14:13.000000"},
+                {"id": "368c5780-ad72-4bcf-a8b6-19e45f4fabar",
+                 "name": "bar",
+                 "created_at": "2016-07-08T14:13:12.000000",
+                 "updated_at": "2016-07-08T15:14:13.000000"},
+            ]
+        }
+        return (200, {}, availability_zones)
+
     def get_os_services(self, **kw):
         services = {
             "services": [
@@ -150,6 +165,24 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
         share = {'share': {'id': 1234, 'name': 'sharename'}}
         return (200, {}, share)
 
+    def get_share_servers_1234(self, **kw):
+        share_servers = {
+            'share_servers': {
+                'id': 1234,
+                'share_network_id': 'fake_network_id_1',
+            },
+        }
+        return (200, {}, share_servers)
+
+    def get_share_servers_5678(self, **kw):
+        share_servers = {
+            'share_servers': {
+                'id': 5678,
+                'share_network_id': 'fake_network_id_2',
+            },
+        }
+        return (200, {}, share_servers)
+
     def get_shares_1111(self, **kw):
         share = {'share': {'id': 1111, 'name': 'share1111'}}
         return (200, {}, share)
@@ -230,6 +263,8 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
             raise AssertionError("Unexpected action: %s" % action)
         return (resp, {}, _body)
 
+    post_snapshots_5678_action = post_snapshots_1234_action
+
     def post_snapshots_manage(self, body, **kw):
         _body = {'snapshot': {'id': 'fake'}}
         resp = 202
@@ -250,6 +285,42 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
         }
         return (200, {}, instances)
 
+    def get_quota_sets_1234(self, *args, **kwargs):
+        quota_set = {
+            'quota_set': {
+                'id': '1234',
+                'shares': 50,
+                'gigabytes': 1000,
+                'snapshots': 50,
+                'snapshot_gigabytes': 1000,
+                'share_networks': 10,
+            }
+        }
+        return (200, {}, quota_set)
+
+    def get_quota_sets_1234_detail(self, *args, **kwargs):
+        quota_set = {
+            'quota_set': {
+                'id': '1234',
+                'shares': {'in_use': 0,
+                           'limit': 50,
+                           'reserved': 0},
+                'gigabytes': {'in_use': 0,
+                              'limit': 10000,
+                              'reserved': 0},
+                'snapshots': {'in_use': 0,
+                              'limit': 50,
+                              'reserved': 0},
+                'snapshot_gigabytes': {'in_use': 0,
+                                       'limit': 1000,
+                                       'reserved': 0},
+                'share_networks': {'in_use': 0,
+                                   'limit': 10,
+                                   'reserved': 0},
+            }
+        }
+        return (200, {}, quota_set)
+
     def get_share_instances(self, **kw):
         return self._share_instances()
 
@@ -367,6 +438,22 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
             assert body[action]['new_size'] is not None
         elif action in ('unmanage', ):
             assert body[action] is None
+        elif action in ('revert', ):
+            assert body[action] is not None
+            assert body[action]['snapshot_id'] is not None
+        elif action in (
+                'migration_cancel', 'migration_complete',
+                'migration_get_progress'):
+            assert body[action] is None
+            if 'migration_get_progress' == action:
+                _body = {'total_progress': 50}
+                return 200, {}, _body
+        elif action in (
+                'os-migrate_share', 'migrate_share',
+                'migration_start'):
+            assert 'host' in body[action]
+        elif action == 'reset_task_state':
+            assert 'task_state' in body[action]
         else:
             raise AssertionError("Unexpected share action: %s" % action)
         return (resp, {}, _body)
@@ -415,6 +502,45 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
     def delete_share_servers_1234(self, **kwargs):
         return (202, {}, None)
 
+    def delete_share_servers_5678(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_security_services_fake_security_service1(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_security_services_fake_security_service2(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_share_networks_fake_share_network1(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_share_networks_fake_share_network2(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_snapshots_fake_snapshot1(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_snapshots_fake_snapshot2(self, **kwargs):
+        return (202, {}, None)
+
+    def post_snapshots_fake_snapshot_force1_action(self, **kwargs):
+        return (202, {}, None)
+
+    def post_snapshots_fake_snapshot_force2_action(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_types_fake_type1(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_types_fake_type2(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_share_servers_fake_share_server1(self, **kwargs):
+        return (202, {}, None)
+
+    def delete_share_servers_fake_share_server2(self, **kwargs):
+        return (202, {}, None)
+
     def put_share_networks_1111(self, **kwargs):
         share_network = {'share_network': {'id': 1111}}
         return (200, {}, share_network)
@@ -507,6 +633,27 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
         }
         return (200, {}, pools)
 
+    def get_scheduler_stats_pools_detail(self, **kw):
+        pools = {
+            'pools': [
+                {
+                    'name': 'host1@backend1#pool1',
+                    'host': 'host1',
+                    'backend': 'backend1',
+                    'pool': 'pool1',
+                    'capabilities': {'qos': True},
+                },
+                {
+                    'name': 'host1@backend1#pool2',
+                    'host': 'host1',
+                    'backend': 'backend1',
+                    'pool': 'pool2',
+                    'capabilities': {'qos': False},
+                }
+            ]
+        }
+        return (200, {}, pools)
+
     def get_consistency_groups_detail(self, **kw):
         consistency_groups = {
             'consistency_groups': [
@@ -672,11 +819,11 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
         return (200, {}, {
             'share_types': [{'id': 1,
                              'name': 'test-type-1',
-                             'extra_specs': {'test': 'test'},
+                             'extra_specs': {'test1': 'test1'},
                              'required_extra_specs': {'test': 'test'}},
                             {'id': 2,
                              'name': 'test-type-2',
-                             'extra_specs': {'test': 'test'},
+                             'extra_specs': {'test1': 'test1'},
                              'required_extra_specs': {'test': 'test'}}]})
 
     def get_types_1(self, **kw):
@@ -715,12 +862,16 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
 
     def post_types(self, body, **kw):
         share_type = body['share_type']
+        required_extra_specs = {
+            "driver_handles_share_servers": share_type[
+                'extra_specs']['driver_handles_share_servers'],
+        }
         return (202, {}, {
             'share_type': {
                 'id': 3,
                 'name': 'test-type-3',
                 'extra_specs': share_type['extra_specs'],
-                'required_extra_specs': share_type['extra_specs'],
+                'required_extra_specs': required_extra_specs,
             }
         })
 
@@ -755,6 +906,53 @@ class FakeHTTPClient(fakes.FakeHTTPClient):
 
     get_types_3_share_type_access = get_types_3_os_share_type_access
 
+    fake_snapshot_instance = {
+        "id": "1234",
+        "snapshot_id": "5678",
+        "status": "error",
+    }
+
+    def get_snapshot_instances(self, **kw):
+        instances = {
+            'snapshot_instances': [
+                self.fake_snapshot_instance,
+            ]
+        }
+        return (200, {}, instances)
+
+    def get_snapshot_instances_detail(self, **kw):
+        instances = {
+            'snapshot_instances': [
+                {
+                    'id': '1234',
+                    'snapshot_id': '5679',
+                    'created_at': 'fake',
+                    'updated_at': 'fake',
+                    'status': 'fake',
+                    'share_id': 'fake',
+                    'share_instance_id': 'fake',
+                    'progress': 'fake',
+                    'provider_location': 'fake',
+                }
+            ]
+        }
+        return (200, {}, instances)
+
+    def get_snapshot_instances_1234(self, **kw):
+        instances = {'snapshot_instance': self.fake_snapshot_instance}
+        return (200, {}, instances)
+
+    def post_snapshot_instances_1234_action(self, body, **kw):
+        _body = None
+        resp = 202
+        assert len(list(body)) == 1
+        action = list(body)[0]
+        if action == 'reset_status':
+            assert 'status' in body.get(action)
+        else:
+            raise AssertionError("Unexpected share action: %s" % action)
+        return (resp, {}, _body)
+
 
 def fake_create(url, body, response_key):
     return {'url': url, 'body': body, 'resp_key': response_key}
@@ -762,3 +960,12 @@ def fake_create(url, body, response_key):
 
 def fake_update(url, body, response_key):
     return {'url': url, 'body': body, 'resp_key': response_key}
+
+
+class FakeQuotaSet(object):
+
+    def __init__(self, dictionary):
+        self.dictionary = dictionary
+
+    def to_dict(self):
+        return self.dictionary
