@@ -1,4 +1,4 @@
-# Copyright 2013 OpenStack LLC.
+# Copyright 2013 OpenStack Foundation
 # Copyright 2014 Mirantis, Inc.
 # All Rights Reserved.
 #
@@ -16,20 +16,26 @@
 
 import ddt
 import fixtures
+import itertools
 import mock
 from oslo_utils import strutils
 import six
 from six.moves.urllib import parse
 
 from manilaclient import client
+from manilaclient.common.apiclient import utils as apiclient_utils
+from manilaclient.common import cliutils
 from manilaclient.common import constants
 from manilaclient import exceptions
-from manilaclient.openstack.common.apiclient import utils as apiclient_utils
-from manilaclient.openstack.common import cliutils
 from manilaclient import shell
 from manilaclient.tests.unit import utils as test_utils
 from manilaclient.tests.unit.v2 import fakes
+from manilaclient.v2 import security_services
 from manilaclient.v2 import share_instances
+from manilaclient.v2 import share_networks
+from manilaclient.v2 import share_servers
+from manilaclient.v2 import share_snapshots
+from manilaclient.v2 import share_types
 from manilaclient.v2 import shell as shell_v2
 
 
@@ -89,14 +95,31 @@ class ShellTest(test_utils.TestCase):
         client.get_client_class = self.old_get_client_class
         super(ShellTest, self).tearDown()
 
-    def run_command(self, cmd):
-        self.shell.main(cmd.split())
+    def run_command(self, cmd, version=None):
+        if version:
+            args = ['--os-share-api-version', version] + cmd.split()
+        else:
+            args = cmd.split()
+        self.shell.main(args)
 
     def assert_called(self, method, url, body=None, **kwargs):
         return self.shell.cs.assert_called(method, url, body, **kwargs)
 
-    def assert_called_anytime(self, method, url, body=None):
-        return self.shell.cs.assert_called_anytime(method, url, body)
+    def assert_called_anytime(self, method, url, body=None,
+                              clear_callstack=True):
+        return self.shell.cs.assert_called_anytime(
+            method, url, body, clear_callstack=clear_callstack)
+
+    def test_availability_zone_list(self):
+        self.run_command('availability-zone-list')
+        self.assert_called('GET', '/availability-zones')
+
+    @mock.patch.object(cliutils, 'print_list', mock.Mock())
+    def test_availability_zone_list_select_column(self):
+        self.run_command('availability-zone-list --columns id,name')
+        self.assert_called('GET', '/availability-zones')
+        cliutils.print_list.assert_called_once_with(
+            mock.ANY, fields=['Id', 'Name'])
 
     def test_service_list(self):
         self.run_command('service-list')
@@ -329,7 +352,7 @@ class ShellTest(test_utils.TestCase):
         cliutils.print_list.assert_called_once_with(
             mock.ANY,
             ['ID', 'Share ID', 'Host', 'Status', 'Availability Zone',
-             'Share Network ID', 'Share Server ID'])
+             'Share Network ID', 'Share Server ID', 'Share Type ID'])
 
     @mock.patch.object(cliutils, 'print_list', mock.Mock())
     def test_share_instance_list_select_column(self):
@@ -431,6 +454,8 @@ class ShellTest(test_utils.TestCase):
                 'extra_specs': {
                     'driver_handles_share_servers': False,
                     'snapshot_support': True,
+                    'create_share_from_snapshot_support': True,
+                    'revert_to_snapshot_support': False,
                 },
                 'share_type_access:is_public': public
             }
@@ -594,6 +619,19 @@ class ShellTest(test_utils.TestCase):
         self.assert_called('POST', '/snapshots/1234/action',
                            body={'unmanage': None})
 
+    def test_revert_to_snapshot(self):
+
+        fake_share_snapshot = type(
+            'FakeShareSnapshot', (object,), {'id': '5678', 'share_id': '1234'})
+        self.mock_object(
+            shell_v2, '_find_share_snapshot',
+            mock.Mock(return_value=fake_share_snapshot))
+
+        self.run_command('revert-to-snapshot 5678')
+
+        self.assert_called('POST', '/shares/1234/action',
+                           body={'revert': {'snapshot_id': '5678'}})
+
     def test_delete(self):
         self.run_command('delete 1234')
         self.assert_called('DELETE', '/shares/1234')
@@ -756,6 +794,35 @@ class ShellTest(test_utils.TestCase):
             'type-create test ' + value,
         )
 
+    @ddt.data('True', 'False')
+    def test_type_create_duplicate_dhss(self, value):
+        self.assertRaises(
+            exceptions.CommandError,
+            self.run_command,
+            'type-create test ' + value +
+            ' --extra-specs driver_handles_share_servers=' + value,
+        )
+
+    @ddt.data(*itertools.product(
+        ['snapshot_support', 'create_share_from_snapshot_support'],
+        ['True', 'False'])
+    )
+    @ddt.unpack
+    def test_type_create_duplicate_switch_and_extra_spec(self, key, value):
+
+        cmd = ('type-create test True --%(key)s %(value)s --extra-specs '
+               '%(key)s=%(value)s' % {'key': key, 'value': value})
+
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+    def test_type_create_duplicate_extra_spec_key(self):
+        self.assertRaises(
+            exceptions.CommandError,
+            self.run_command,
+            'type-create test True --extra-specs'
+            ' a=foo1 a=foo2',
+        )
+
     @ddt.unpack
     @ddt.data({'expected_bool': True, 'text': 'true'},
               {'expected_bool': True, 'text': '1'},
@@ -769,6 +836,8 @@ class ShellTest(test_utils.TestCase):
                 "extra_specs": {
                     "driver_handles_share_servers": expected_bool,
                     "snapshot_support": True,
+                    "create_share_from_snapshot_support": True,
+                    "revert_to_snapshot_support": False,
                 }
             }
         }
@@ -784,7 +853,30 @@ class ShellTest(test_utils.TestCase):
           [{'expected_bool': False, 'text': v}
            for v in ('false', 'False', '0', 'FALSE', 'fAlSe')])
     )
-    def test_create_with_snapshot_support(self, expected_bool, text):
+    def test_type_create_with_snapshot_support(self, expected_bool, text):
+        expected = {
+            "share_type": {
+                "name": "test",
+                "share_type_access:is_public": True,
+                "extra_specs": {
+                    "snapshot_support": expected_bool,
+                    "driver_handles_share_servers": False,
+                }
+            }
+        }
+        self.run_command('type-create test false --snapshot-support ' + text)
+
+        self.assert_called('POST', '/types', body=expected)
+
+    @ddt.unpack
+    @ddt.data({'expected_bool': True,
+               'snapshot_text': 'true',
+               'replication_type': 'readable'},
+              {'expected_bool': False,
+               'snapshot_text': 'false',
+               'replication_type': 'writable'})
+    def test_create_with_extra_specs(self, expected_bool, snapshot_text,
+                                     replication_type):
         expected = {
             "share_type": {
                 "name": "test",
@@ -792,20 +884,95 @@ class ShellTest(test_utils.TestCase):
                 "extra_specs": {
                     "driver_handles_share_servers": False,
                     "snapshot_support": expected_bool,
+                    "create_share_from_snapshot_support": True,
+                    "revert_to_snapshot_support": False,
+                    "replication_type": replication_type,
                 }
             }
         }
 
-        self.run_command('type-create test false --snapshot-support ' + text)
+        self.run_command('type-create test false --extra-specs'
+                         ' snapshot_support=' + snapshot_text +
+                         ' replication_type=' + replication_type)
+
+        self.assert_called('POST', '/types', body=expected)
+
+    @ddt.unpack
+    @ddt.data(
+        *([{'expected_bool': True, 'text': v}
+           for v in ('true', 'True', '1', 'TRUE', 'tRuE')] +
+          [{'expected_bool': False, 'text': v}
+           for v in ('false', 'False', '0', 'FALSE', 'fAlSe')])
+    )
+    def test_type_create_with_create_share_from_snapshot_support(
+            self, expected_bool, text):
+        expected = {
+            "share_type": {
+                "name": "test",
+                "share_type_access:is_public": True,
+                "extra_specs": {
+                    "driver_handles_share_servers": False,
+                    "snapshot_support": True,
+                    "create_share_from_snapshot_support": expected_bool,
+                    "revert_to_snapshot_support": False,
+                }
+            }
+        }
+
+        self.run_command('type-create test false --snapshot-support true '
+                         '--create-share-from-snapshot-support ' + text)
+
+        self.assert_called('POST', '/types', body=expected)
+
+    @ddt.data('snapshot_support', 'create_share_from_snapshot_support')
+    def test_type_create_invalid_switch_value(self, value):
+        self.assertRaises(
+            exceptions.CommandError,
+            self.run_command,
+            'type-create test false --%s fake' % value,
+        )
+
+    @ddt.data('snapshot_support', 'create_share_from_snapshot_support')
+    def test_type_create_invalid_extra_spec_value(self, value):
+        self.assertRaises(
+            exceptions.CommandError,
+            self.run_command,
+            'type-create test false --extra-specs %s=fake' % value,
+        )
+
+    @ddt.unpack
+    @ddt.data(
+        *([{'expected_bool': True, 'text': v}
+           for v in ('true', 'True', '1', 'TRUE', 'tRuE')] +
+          [{'expected_bool': False, 'text': v}
+           for v in ('false', 'False', '0', 'FALSE', 'fAlSe')])
+    )
+    def test_type_create_with_revert_to_snapshot_support(
+            self, expected_bool, text):
+        expected = {
+            "share_type": {
+                "name": "test",
+                "share_type_access:is_public": True,
+                "extra_specs": {
+                    "driver_handles_share_servers": False,
+                    "snapshot_support": True,
+                    "create_share_from_snapshot_support": True,
+                    "revert_to_snapshot_support": expected_bool,
+                }
+            }
+        }
+
+        self.run_command('type-create test false --snapshot-support true '
+                         '--revert-to-snapshot-support ' + text)
 
         self.assert_called('POST', '/types', body=expected)
 
     @ddt.data('fake', 'FFFalse', 'trueee')
-    def test_type_create_invalid_snapshot_support_value(self, value):
+    def test_type_create_invalid_revert_to_snapshot_support_value(self, value):
         self.assertRaises(
             exceptions.CommandError,
             self.run_command,
-            'type-create test false --snapshot-support ' + value,
+            'type-create test false --revert-to-snapshot-support ' + value,
         )
 
     @ddt.data('--is-public', '--is_public')
@@ -947,19 +1114,16 @@ class ShellTest(test_utils.TestCase):
         {},
         {'--name': 'fake_name'},
         {'--description': 'fake_description'},
-        {'--nova_net_id': 'fake_nova_net_id'},
         {'--neutron_net_id': 'fake_neutron_net_id'},
         {'--neutron_subnet_id': 'fake_neutron_subnet_id'},
         {'--description': 'fake_description',
          '--name': 'fake_name',
          '--neutron_net_id': 'fake_neutron_net_id',
-         '--neutron_subnet_id': 'fake_neutron_subnet_id',
-         '--nova_net_id': 'fake_nova_net_id'})
+         '--neutron_subnet_id': 'fake_neutron_subnet_id'})
     def test_share_network_create(self, data):
         cmd = 'share-network-create'
         for k, v in data.items():
             cmd += ' ' + k + ' ' + v
-
         self.run_command(cmd)
 
         self.assert_called('POST', '/share-networks')
@@ -967,24 +1131,21 @@ class ShellTest(test_utils.TestCase):
     @ddt.data(
         {'--name': 'fake_name'},
         {'--description': 'fake_description'},
-        {'--nova_net_id': 'fake_nova_net_id'},
         {'--neutron_net_id': 'fake_neutron_net_id'},
         {'--neutron_subnet_id': 'fake_neutron_subnet_id'},
         {'--description': 'fake_description',
          '--name': 'fake_name',
          '--neutron_net_id': 'fake_neutron_net_id',
-         '--neutron_subnet_id': 'fake_neutron_subnet_id',
-         '--nova_net_id': 'fake_nova_net_id'},
+         '--neutron_subnet_id': 'fake_neutron_subnet_id'},
         {'--name': '""'},
         {'--description': '""'},
-        {'--nova_net_id': '""'},
         {'--neutron_net_id': '""'},
         {'--neutron_subnet_id': '""'},
         {'--description': '""',
          '--name': '""',
          '--neutron_net_id': '""',
          '--neutron_subnet_id': '""',
-         '--nova_net_id': '""'},)
+         },)
     def test_share_network_update(self, data):
         cmd = 'share-network-update 1111'
         expected = dict()
@@ -1084,19 +1245,6 @@ class ShellTest(test_utils.TestCase):
                 mock.ANY,
                 fields=['id', 'name'])
 
-    @mock.patch.object(cliutils, 'print_list', mock.Mock())
-    def test_share_network_list_nova_net_id_aliases(self):
-        for command in ['--nova-net-id', '--nova-net_id',
-                        '--nova_net-id', '--nova_net_id']:
-            self.run_command('share-network-list %s fake-id' % command)
-            self.assert_called(
-                'GET',
-                '/share-networks/detail?nova_net_id=fake-id',
-            )
-            cliutils.print_list.assert_called_with(
-                mock.ANY,
-                fields=['id', 'name'])
-
     @mock.patch.object(cliutils, 'print_list', mock.Mock())
     def test_share_network_list_neutron_net_id_aliases(self):
         for command in ['--neutron-net-id', '--neutron-net_id',
@@ -1254,10 +1402,6 @@ class ShellTest(test_utils.TestCase):
             mock.ANY,
             fields=['Id', 'Host', 'Status'])
 
-    def test_share_server_delete(self):
-        self.run_command('share-server-delete 1234')
-        self.assert_called('DELETE', '/share-servers/1234')
-
     def test_create_share(self):
         # Use only required fields
         self.run_command("create nfs 1")
@@ -1362,7 +1506,8 @@ class ShellTest(test_utils.TestCase):
         self.run_command("access-list 1111")
         cliutils.print_list.assert_called_with(
             mock.ANY,
-            ['id', 'access_type', 'access_to', 'access_level', 'state'])
+            ['id', 'access_type', 'access_to', 'access_level', 'state',
+             'access_key'])
 
     @mock.patch.object(cliutils, 'print_list', mock.Mock())
     def test_access_list_select_column(self):
@@ -1526,6 +1671,44 @@ class ShellTest(test_utils.TestCase):
             mock.ANY,
             fields=["Name", "Host", "Backend", "Pool"])
 
+    @mock.patch.object(cliutils, 'print_dict', mock.Mock())
+    def test_quota_show(self):
+        self.run_command('quota-show --tenant 1234')
+        self.assert_called(
+            'GET',
+            '/quota-sets/1234',
+        )
+        cliutils.print_dict.assert_called_once_with(mock.ANY)
+
+    @mock.patch.object(cliutils, 'print_dict', mock.Mock())
+    def test_quota_show_with_detail(self):
+        self.run_command('quota-show --tenant 1234 --detail')
+        self.assert_called(
+            'GET',
+            '/quota-sets/1234/detail',
+        )
+        cliutils.print_dict.assert_called_once_with(mock.ANY)
+
+    @mock.patch.object(cliutils, 'print_dict', mock.Mock())
+    def test_quota_show_with_user_id(self):
+        self.run_command('quota-show --tenant 1234 --user 1111')
+        self.assert_called(
+            'GET',
+            '/quota-sets/1234?user_id=1111',
+        )
+        cliutils.print_dict.assert_called_once_with(mock.ANY)
+
+    @mock.patch.object(cliutils, 'print_list', mock.Mock())
+    def test_pool_list_with_detail(self):
+        self.run_command('pool-list --detail')
+        self.assert_called(
+            'GET',
+            '/scheduler-stats/pools/detail?backend=.%2A&host=.%2A&pool=.%2A',
+        )
+        cliutils.print_list.assert_called_with(
+            mock.ANY,
+            fields=["Name", "Host", "Backend", "Pool", 'Capabilities'])
+
     @mock.patch.object(cliutils, 'print_list', mock.Mock())
     def test_pool_list_select_column(self):
         self.run_command('pool-list --columns name,host')
@@ -1537,13 +1720,34 @@ class ShellTest(test_utils.TestCase):
             mock.ANY,
             fields=["Name", "Host"])
 
+    @ddt.data(({"key1": "value1",
+               "key2": "value2"},
+               {"key1": "value1",
+               "key2": "value2"}),
+              ({"key1": {"key11": "value11", "key12": "value12"},
+                "key2": {"key21": "value21"}},
+               {"key1": "key11 = value11\nkey12 = value12",
+                "key2": "key21 = value21"}),
+              ({}, {}))
+    @ddt.unpack
+    @mock.patch.object(cliutils, 'print_dict', mock.Mock())
+    def test_quota_set_pretty_show(self, value, expected):
+        fake_quota_set = fakes.FakeQuotaSet(value)
+
+        shell_v2._quota_set_pretty_show(fake_quota_set)
+        cliutils.print_dict.assert_called_with(expected)
+
+    @ddt.data('--share-type test_type', '--share_type test_type',
+              '--share-type-id 0123456789', '--share_type_id 0123456789')
     @mock.patch.object(cliutils, 'print_list', mock.Mock())
-    def test_pool_list_with_filters(self):
-        self.run_command(
-            'pool-list --host host1 --backend backend1 --pool pool1')
+    def test_pool_list_with_filters(self, param):
+        cmd = ('pool-list --host host1 --backend backend1 --pool pool1' + ' ' +
+               param)
+        self.run_command(cmd)
         self.assert_called(
             'GET',
-            '/scheduler-stats/pools?backend=backend1&host=host1&pool=pool1',
+            '/scheduler-stats/pools?backend=backend1&host=host1&'
+            'pool=pool1&share_type=%s' % param.split()[-1],
         )
         cliutils.print_list.assert_called_with(
             mock.ANY,
@@ -1765,6 +1969,15 @@ class ShellTest(test_utils.TestCase):
         self.assert_called(
             'GET', '/share-replicas/detail?share_id=fake-share-id')
 
+    @mock.patch.object(cliutils, 'print_list', mock.Mock())
+    def test_share_replica_list_select_column(self):
+        self.run_command('share-replica-list --columns id,status')
+
+        self.assert_called('GET', '/share-replicas/detail')
+
+        cliutils.print_list.assert_called_once_with(
+            mock.ANY, ['Id', 'Status'])
+
     @ddt.data(
         'fake-share-id --az fake-az',
         'fake-share-id --availability-zone fake-az --share-network '
@@ -1825,3 +2038,196 @@ class ShellTest(test_utils.TestCase):
         self.assert_called(
             'POST', '/share-replicas/1234/action',
             body={action_name: {attr: 'xyzzyspoon!'}})
+
+    def test_snapshot_instance_list_all(self):
+        self.run_command('snapshot-instance-list')
+        self.assert_called('GET', '/snapshot-instances')
+
+    def test_snapshot_instance_list_all_detail(self):
+        self.run_command('snapshot-instance-list --detail True')
+        self.assert_called('GET', '/snapshot-instances/detail')
+
+    @mock.patch.object(cliutils, 'print_list', mock.Mock())
+    def test_snapshot_instance_list_select_column(self):
+        self.run_command('snapshot-instance-list --columns id,status')
+        self.assert_called('GET', '/snapshot-instances')
+        cliutils.print_list.assert_called_once_with(
+            mock.ANY, ['Id', 'Status'])
+
+    @mock.patch.object(shell_v2, '_find_share_snapshot', mock.Mock())
+    def test_snapshot_instance_list_for_snapshot(self):
+        fsnapshot = type('FakeSansphot', (object,),
+                         {'id': 'fake-snapshot-id'})
+        shell_v2._find_share_snapshot.return_value = fsnapshot
+        cmd = 'snapshot-instance-list --snapshot %s'
+        self.run_command(cmd % fsnapshot.id)
+
+        self.assert_called(
+            'GET', '/snapshot-instances?snapshot_id=fake-snapshot-id')
+
+    def test_snapshot_instance_show(self):
+        self.run_command('snapshot-instance-show 1234')
+        self.assert_called('GET', '/snapshot-instances/1234')
+
+    def test_snapshot_instance_reset_state(self):
+        self.run_command('snapshot-instance-reset-state 1234')
+        expected = {'reset_status': {'status': 'available'}}
+        self.assert_called('POST', '/snapshot-instances/1234/action',
+                           body=expected)
+
+    def test_migration_start(self):
+        command = ("migration-start --force-host-assisted-migration True "
+                   "--new-share-network 1111 --new-share-type 1 1234 "
+                   "host@backend#pool --writable False --nondisruptive True "
+                   "--preserve-metadata False --preserve-snapshots True")
+        self.run_command(command)
+        expected = {'migration_start': {
+            'host': 'host@backend#pool',
+            'force_host_assisted_migration': 'True',
+            'preserve-metadata': 'False',
+            'writable': 'False',
+            'nondisruptive': 'True',
+            'preserve_snapshots': 'True',
+            'new_share_network_id': '1111',
+            'new_share_type_id': '1'
+        }}
+        self.assert_called('POST', '/shares/1234/action', body=expected)
+
+    @ddt.data('migration-complete', 'migration-get-progress',
+              'migration-cancel')
+    def test_migration_others(self, method):
+        command = ' '.join((method, '1234'))
+        self.run_command(command)
+        expected = {method.replace('-', '_'): None}
+        self.assert_called('POST', '/shares/1234/action', body=expected)
+
+    @ddt.data('migration_error', 'migration_success', None)
+    def test_reset_task_state(self, param):
+        command = ' '.join(('reset-task-state --state', six.text_type(param),
+                            '1234'))
+        self.run_command(command)
+        expected = {'reset_task_state': {'task_state': param}}
+        self.assert_called('POST', '/shares/1234/action', body=expected)
+
+    @ddt.data(('fake_security_service1', ),
+              ('fake_security_service1', 'fake_security_service2'))
+    def test_security_service_delete(self, ss_ids):
+        fake_security_services = [
+            security_services.SecurityService('fake', {'id': ss_id}, True)
+            for ss_id in ss_ids
+        ]
+        self.mock_object(
+            shell_v2, '_find_security_service',
+            mock.Mock(side_effect=fake_security_services))
+
+        self.run_command('security-service-delete %s' % ' '.join(ss_ids))
+
+        shell_v2._find_security_service.assert_has_calls([
+            mock.call(self.shell.cs, ss_id) for ss_id in ss_ids
+        ])
+        for ss in fake_security_services:
+            self.assert_called_anytime(
+                'DELETE', '/security-services/%s' % ss.id,
+                clear_callstack=False)
+
+    @ddt.data(('fake_share_network1', ),
+              ('fake_share_network1', 'fake_share_network1'))
+    def test_share_network_delete(self, sn_ids):
+        fake_share_networks = [
+            share_networks.ShareNetwork('fake', {'id': sn_id}, True)
+            for sn_id in sn_ids
+        ]
+        self.mock_object(
+            shell_v2, '_find_share_network',
+            mock.Mock(side_effect=fake_share_networks))
+
+        self.run_command('share-network-delete %s' % ' '.join(sn_ids))
+
+        shell_v2._find_share_network.assert_has_calls([
+            mock.call(self.shell.cs, sn_id) for sn_id in sn_ids
+        ])
+        for sn in fake_share_networks:
+            self.assert_called_anytime(
+                'DELETE', '/share-networks/%s' % sn.id,
+                clear_callstack=False)
+
+    @ddt.data(('fake_snapshot1', ), ('fake_snapshot1', 'fake_snapshot2'))
+    def test_snapshot_delete(self, snapshot_ids):
+        fake_snapshots = [
+            share_snapshots.ShareSnapshot('fake', {'id': snapshot_id}, True)
+            for snapshot_id in snapshot_ids
+        ]
+        self.mock_object(
+            shell_v2, '_find_share_snapshot',
+            mock.Mock(side_effect=fake_snapshots))
+
+        self.run_command('snapshot-delete %s' % ' '.join(snapshot_ids))
+
+        shell_v2._find_share_snapshot.assert_has_calls([
+            mock.call(self.shell.cs, s_id) for s_id in snapshot_ids
+        ])
+        for snapshot in fake_snapshots:
+            self.assert_called_anytime(
+                'DELETE', '/snapshots/%s' % snapshot.id,
+                clear_callstack=False)
+
+    @ddt.data(('1234', ), ('1234', '5678'))
+    def test_snapshot_force_delete(self, snapshot_ids):
+        fake_snapshots = [
+            share_snapshots.ShareSnapshot('fake', {'id': snapshot_id}, True)
+            for snapshot_id in snapshot_ids
+        ]
+        self.mock_object(
+            shell_v2, '_find_share_snapshot',
+            mock.Mock(side_effect=fake_snapshots))
+
+        self.run_command('snapshot-force-delete %s' % ' '.join(snapshot_ids))
+
+        shell_v2._find_share_snapshot.assert_has_calls([
+            mock.call(self.shell.cs, s_id) for s_id in snapshot_ids
+        ])
+        for snapshot in fake_snapshots:
+            self.assert_called_anytime(
+                'POST', '/snapshots/%s/action' % snapshot.id,
+                {'force_delete': None},
+                clear_callstack=False)
+
+    @ddt.data(('fake_type1', ), ('fake_type1', 'fake_type2'))
+    def test_share_type_delete(self, type_ids):
+        fake_share_types = [
+            share_types.ShareType('fake', {'id': type_id}, True)
+            for type_id in type_ids
+        ]
+        self.mock_object(
+            shell_v2, '_find_share_type',
+            mock.Mock(side_effect=fake_share_types))
+
+        self.run_command('type-delete %s' % ' '.join(type_ids))
+
+        shell_v2._find_share_type.assert_has_calls([
+            mock.call(self.shell.cs, t_id) for t_id in type_ids
+        ])
+        for fake_share_type in fake_share_types:
+            self.assert_called_anytime(
+                'DELETE', '/types/%s' % fake_share_type.id,
+                clear_callstack=False)
+
+    @ddt.data(('1234', ), ('1234', '5678'))
+    def test_share_server_delete(self, server_ids):
+        fake_share_servers = [
+            share_servers.ShareServer('fake', {'id': server_id}, True)
+            for server_id in server_ids
+        ]
+        self.mock_object(
+            shell_v2, '_find_share_server',
+            mock.Mock(side_effect=fake_share_servers))
+
+        self.run_command('share-server-delete %s' % ' '.join(server_ids))
+
+        shell_v2._find_share_server.assert_has_calls([
+            mock.call(self.shell.cs, s_id) for s_id in server_ids
+        ])
+        for server in fake_share_servers:
+            self.assert_called_anytime(
+                'DELETE', '/share-servers/%s' % server.id,
+                clear_callstack=False)
